

import NwfValidator from '../nwfValidator';

class IsEmailValidatorController extends NwfValidator {

    static $inject = ['$element'];

    constructor(private $element: ng.IAugmentedJQuery) {
        super($element);
        this.myEl.setAttribute('type', 'email');
        this.myEl.setAttribute('multiple', 'multiple');
        this.invalidMessage = this.invalidMessage || 'Please enter a valid email';
        this.emptyMessage = this.emptyMessage || 'You have to provide a value';
    }

    invalidHandler() {
        if (this.myEl.validity.valueMissing && !this.myEl.validity.badInput) {
            this.myEl.setCustomValidity(this.emptyMessage);
        }
        else if (!this.myEl.validity.valid) {
            // if a semi-colon exists in the value of the textbox, we display comma-separated error.
            if (this.myEl.value.indexOf(';') > -1) {
                this.myEl.setCustomValidity('Multiple email addresses should be comma separated');
            } else {
                this.myEl.setCustomValidity(this.invalidMessage);
            }
        } else {
            this.myEl.setCustomValidity('');
        }
    }
}

export class IsEmailValidator {

    static NAME: string = 'isEmail';
    bindToController = {
        emptyMessage: '@?',
        invalidMessage: '@?'
    };
    restrict = 'A';
    controller = IsEmailValidatorController;
    controllerAs = 'emailCtrl';
}

// export factory function
export function IsEmailValidatorFactory(): IsEmailValidator {
    return new IsEmailValidator();
}