
import * as angular from 'angular';


export default class PaginatorController{

    static $inject = ['$scope','$timeout'];

    paginatorProperties: any;
    title: string;
    totalRecords: number;

    constructor(private $scope, private $timeout){

        this.paginatorProperties = {
            title: $scope.title,
            page : '',
            start : '',
            end: '',
            wrongInput :false,
            totalPages: '',
            totalItems: '',
            offset: '',
            limit: ''
        };

        // Watch on total Records coming from parent controller of the page.
        $scope.$watch('totalRecords',(newValue) => {
            this.paginatorProperties.totalItems = parseInt(newValue, 10);

            //setting the parameters to be displayed.
            this.setTemplateParameters();
        });

        // Watch on title coming from parent controller of the page.
        $scope.$watch('title',(newValue)=>{
            this.paginatorProperties.title = newValue;
        });

        // Watch on offset as it can be changed from sort parameter.
        $scope.$watch('offset',(newVal,oldVal)=>{
            if(newVal === 0){
                //resetting everything on the template when this is called.
                this.paginatorProperties.totalItems = parseInt($scope.totalRecords, 10);

                //setting the parameters to be displayed.
                this.setTemplateParameters();
            }
        });

    }

    //Method that sets all the parameters values displayed on paginator template.
    setTemplateParameters(){

        this.paginatorProperties.limit = this.$scope.limit;
        this.paginatorProperties.offset = this.$scope.offset;
        this.paginatorProperties.totalPages = Math.ceil(this.paginatorProperties.totalItems / this.paginatorProperties.limit);

        // Page number and start value should be zero when there are no records returned.
        if(this.paginatorProperties.totalItems === 0){
            this.paginatorProperties.start = 0;
            this.paginatorProperties.page = 0;
        }
        else{
            this.paginatorProperties.start = 1;
            this.paginatorProperties.page = 1;
        }

        this.paginatorProperties.end = Math.min(this.paginatorProperties.limit, this.paginatorProperties.totalItems);
    }

    //method that gets called when the user selects a particular button on paginator. inputs type of button clicked.
    selectPage(type) {

        switch (type) {
            case 'first':
                this.paginatorProperties.page = 1;
                this.paginatorProperties.start = 1;
                this.paginatorProperties.end = Math.min(this.paginatorProperties.limit, this.paginatorProperties.totalItems);
                this.paginatorProperties.wrongInput = false;

                break;
            case 'previous':
                //Disabling setting of the offset if page entered/no is less than 1 or greater than total page count.
                if(!this.paginatorProperties.wrongInput) {
                    if (this.paginatorProperties.page <= 1 || this.paginatorProperties.page > this.paginatorProperties.totalPages) {
                        break;
                    }
                    this.paginatorProperties.page = parseInt(this.paginatorProperties.page, 10) - 1;
                    this.paginatorProperties.start = ((this.paginatorProperties.page - 1) * this.paginatorProperties.limit) + 1;
                    this.paginatorProperties.end = this.paginatorProperties.page * this.paginatorProperties.limit;
                }
                break;
            case 'next':
                //Disabling setting of the offset if page entered/no is greater than total page count.
                if(!this.paginatorProperties.wrongInput) {
                    if (this.paginatorProperties.page >= this.paginatorProperties.totalPages) {
                        break;
                    }
                    this.paginatorProperties.start = (this.paginatorProperties.page * this.paginatorProperties.limit) + 1;
                    this.paginatorProperties.page = parseInt(this.paginatorProperties.page, 10) + 1;

                    if (this.paginatorProperties.page === this.paginatorProperties.totalPages) {
                        this.paginatorProperties.end = this.paginatorProperties.totalItems;
                    } else {
                        this.paginatorProperties.end = this.paginatorProperties.page * this.paginatorProperties.limit;
                    }
                }
                break;
            case 'last':
                this.paginatorProperties.page = this.paginatorProperties.totalPages;
                this.paginatorProperties.start = ((this.paginatorProperties.page - 1) * this.paginatorProperties.limit) + 1;
                this.paginatorProperties.end = this.paginatorProperties.totalItems;
                this.paginatorProperties.wrongInput = false;
                break;
        }

        //Setting the offset only if its a valid page number.ie.wrongInput=false.
        if (!this.paginatorProperties.wrongInput) {
            this.paginatorProperties.offset = this.paginatorProperties.start - 1;
            this.$scope.paramsCtrl.params.offset = this.paginatorProperties.offset;
        }
    }

    //this method executes on every key entered in paginator textbox.
    valueEntered(e,p){

        //this executes only when Enter Key (keycode=13) is pressed and the value is not empty.
        if(e.keyCode === 13 && p!=='') {

            //converting string input value to integer.
            var pageNo = parseInt(p, 10);

            //setting wrong input to true when entered page number > 1 or greater than total page count or its NaN.
            if (pageNo > this.paginatorProperties.totalPages || pageNo < 1 || !pageNo) {
                this.paginatorProperties.wrongInput = true;
            } else {
                this.paginatorProperties.wrongInput = false;
                this.paginatorProperties.start = (pageNo - 1) * (this.paginatorProperties.limit) + 1;
                if (pageNo == this.paginatorProperties.totalPages) {
                    this.paginatorProperties.end = this.paginatorProperties.totalItems;
                } else {
                    this.paginatorProperties.end = pageNo * this.paginatorProperties.limit;
                }
                this.paginatorProperties.offset = this.paginatorProperties.start - 1;
                this.$scope.paramsCtrl.params.offset = parseInt(this.paginatorProperties.offset, 10);


                //Angular Timeout is used to enable the text-box blur event skip the $apply cycle of value entered in the text-box.
                this.$timeout(function () {
                    angular.element('.paginator-input').blur();
                });
            }
        }

    }

}