import * as angular from 'angular';

class NwfLoadingIndicatorController {

    myElement: any;
    hideCls: string;
    showCls: string;

    static $inject = ['$element', '$rootScope'];

    constructor($element: angular.IAugmentedJQuery, $rootScope: angular.IRootScopeService) {

        this.myElement = $element.get(0); this.hideCls = 'hide'; this.showCls = 'show';

        /* Now I connect my internal controller Events bound to me */
        $rootScope.$on('loadingIndicator', this.showOrHide.bind(this));

        /* when a state transition starts, I go down. */
        $rootScope.$on('$stateChangeStart', (event, toState, toParams, fromState, fromParams) => {
            this.hide();
        });
    }


    show(overlayStyle) {
        this.resetPointerEvents();
        this.myElement.classList.remove(this.hideCls);
        /* default styling is full, we add overlay-partial class */
        var mainViewContainer: any = document.getElementById('mainViewContainer');
        var mainContent: any = document.querySelector('main');
        if (mainViewContainer && mainContent) {
            if (overlayStyle == 'full') {
                mainViewContainer.style.pointerEvents = 'none';
            } else if (overlayStyle == 'partial') {
                mainContent.style.pointerEvents = 'none';
            }
        }
    }

    hide() {
        this.resetPointerEvents();
        this.myElement.classList.add(this.hideCls);
    }

    resetPointerEvents() {
        var mainViewContainer = document.getElementById('mainViewContainer');
        var mainContent = document.querySelector('main');
        if (mainViewContainer) {
            mainViewContainer.removeAttribute('style');
        }
        if (mainContent) {
            mainContent.removeAttribute('style');
        }
    }

    showOrHide(thisObject, showOrHide, overlayStyle) {
        overlayStyle = overlayStyle || 'partial';
        if (showOrHide == this.showCls) {
            this.show(overlayStyle);
        } else if (showOrHide == this.hideCls) {
            this.hide();
        } else {
            throw Error('You told the loadingIndicator to do something other than show or hide');
        }
    }
}


export class NwfLoadingIndicator implements angular.IDirective {
    static NAME: string = 'nwfLoadingIndicator';
    restrict = 'E';
    scope = false;
    controller = NwfLoadingIndicatorController;
}


// export factory function
export function NwfLoadingIndicatorFactory(): NwfLoadingIndicator {
    return new NwfLoadingIndicator();
}
