///<amd-dependency path='text!../page-alert/pageAlert-template.html' name='pageAlertTemplate' />

declare var pageAlertTemplate;
import * as angular from 'angular';


class NwfPageAlertController {

    message: string;
    severity: string;
    domNode: HTMLElement;
    offScreen: string;
    stateClass: string;
    iconClass: string;
    stateChangeStartTransition: any;

    static $inject = ['$rootScope', '$scope', '$timeout', '$compile', '$element', '$sce'];

    constructor(private $rootScope: angular.IRootScopeService, private $scope: angular.IScope, private $timeout: angular.ITimeoutService,
        private $compile: angular.ICompileService, private $element: angular.IAugmentedJQuery, private $sce: angular.ISCEService) {

        this.domNode = $element.get(0);
        this.offScreen = '-100000em';

        /* Now I connect my internal controller Events bound to me */
        $rootScope.$on('pageAlertShowMessage', this.showMessage.bind(this));
        $rootScope.$on('pageAlertHide', this.close.bind(this));

        /* Set my initial position offScreen */
        this.domNode.style.left = this.offScreen;

    }

    /* To call this outside me, use e.g. angular.element(document.body).scope().$emit('pageAlertShowMessage', 'My answer is this - nothing.', 'critical'); */
    showMessage(event, message, severity, fade = false) {

        /* First I test if the message has a valid html start and end tag, if not, I do it for them */
        var hasStartAndEndTag = message.charAt(0) === '<' && message.charAt(message.length) === '>';
        if (!hasStartAndEndTag) {
            message = '<span>' + message + '</span>';
        }

        var compiled = this.$compile(message)(this.$scope);
        this.message = this.$sce.trustAsHtml(compiled.html());
        this.severity = severity;
        this.domNode.style.left = '';
        this.$element.addClass('in');

        var stateClass = 'is-ok', iconClass = 'icon-OkCircleWithCheck';
        if (severity == 'critical') {
            stateClass = 'is-critical';
            iconClass = 'icon-Critical-Full';
        } else if (severity == 'warning') {
            stateClass = 'is-warning';
            iconClass = 'icon-Warning';
        } else if (severity == 'ok') {
            stateClass = 'is-ok';
            iconClass = 'icon-OkCircleWithCheck';
        } else if (severity == 'error') {
            stateClass = 'is-error';
            iconClass = 'icon-Error-Full';
        } else if (severity == 'info') {
            stateClass = 'is-info';
            iconClass = 'icon-Info';
        }
        this.stateClass = stateClass;
        this.iconClass = iconClass;

        /* I don't like this but I don't know how else yet to get it to update */
        this.$timeout(() => {
            this.$scope.$digest();
        });

        //Fade after timeout
        if (fade) {
            this.$timeout(this.close.bind(this), 7500);
        }

        //I come down when transition happens
        this.stateChangeStartTransition = this.$rootScope.$on('$stateChangeStart', () => {
            this.close();
        });

    }

    /* To call this outside me, use e.g. angular.element(document.body).scope().$emit('pageAlertHide'); */
    close() {
        this.$element.get(0).style.left = this.offScreen;
        this.$element.removeClass('in');
        /* Now I de-register the listener for efficiency */
        if (angular.isFunction(this.stateChangeStartTransition)) {
            this.stateChangeStartTransition();
        }
        /* I don't like this but I don't know how else yet to get it to update */
        this.$timeout(() => {
            this.$scope.$digest();
        });
    }


}


export class NwfPageAlert implements angular.IDirective {
    static NAME = 'nwfPageAlert';
    restrict = 'E';
    replace = true;
    template = pageAlertTemplate;
    controller = NwfPageAlertController;
    bindToController = {
        severity: '@',
        message: '@'
    };
    controllerAs = 'pageAlertCtrl';
}

export function NwfPageAlertFactory(): NwfPageAlert {
    return new NwfPageAlert();
}